package student;


import java.util.*;

/** An instance is a min-heap of distinct elements of type E.
  *  Priorities are double values. Since it's a min-heap, the value
  *  with the smallest priority is at the root of the heap. */
public class Heap<E>{
    
    private int size; // number of elements in the heap
    
    /** The heap invariant is given below. Note that / denotes int division.
      * 
      *  b[0..size-1] is viewed as a min-heap: 
      *  1. Each array element in b[0..size-1] contains a value of the heap.
      *  2. The children of each b[i] are b[2i+1] and b[2i+2].
      *  3. The parent of each b[i] (except b[0]) is b[(i-1)/2].
      *  4. The priority of the parent of each b[i] is <= the priority of b[i].
      *  5. Priorities for the b[i] used for the comparison in point 4
      *     are given in map. map contains one entry for each element of
      *     the heap, so map and b have the same size.
      *     For each element e in the heap, the map entry contains in the
      *     Prindex object the priority of e and its index in b.
      */
    private ArrayList<E> b;
    private HashMap<E, Prindex> map= new HashMap<E, Prindex>();
    
    /** Constructor: an empty heap. */
    public Heap() {
        b= new ArrayList<E>();
    }
    
    /** Return a string that gives this heap, in the format:
      * [item0:priority0, item1:priority1, ..., item(N-1):priority(N-1)]
      * Thus, the list is delimited by '['  and ']' and ", " (i.e. a
      * comma and a space char) separate adjacent items. */
    @Override public String toString() {
        String s= "[";
        for (E t : b) {
            if (s.length() > 1) {
                s = s + ", ";
            }
            s = s + t + ":" + map.get(t).priority;
        }
        return s + "]";
    }
    
    /** Return a string that gives the priorities in this heap,
      * in the format: [priority0, priority1, ..., priority(N-1)]
      * Thus, the list is delimited by '['  and ']' and ", " (i.e. a
      * comma and a space char) separate adjacent items. */
    public String toStringPriorities() {
        String s= "[";
        for (E t : b) {
            if (s.length() > 1) {
                s = s + ", ";
            }
            s = s + map.get(t).priority;
        }
        return s + "]";
    }
    
    /** Return the number of elements in this heap.
      *  This operation takes constant time. */
    public int size() {
        return size;
    }
    
    /** Add e with priority p to the heap.
      *  Throw an illegalArgumentException if e is already in the heap.
      *  The expected time is logarithmic and the worst-case time is linear
      *  in the size of the heap. */
    public void add(E e, double p) throws IllegalArgumentException {
        // TODO 1: Do add and bubbleUp together.
        // This is the ONLY method that should create a Prindex object.
        // Our solution to Heap will be available on the Piazza on the
        // morning of 23 April.
    }
    
    /** Return the element of this heap with lowest priority, without
      *  changing the heap. This operation takes constant time.
      *  Throw a HeapException if the heap is empty. */
    public E peek() {
        // TODO 2: Do peek.
        // Our solution to Heap will be available on the Piazza on the morning
        // of 23 April.
            return null;
    }
    
    /** Remove and return the element of this heap with lowest priority.
      *  The expected time is logarithmic and the worst-case time is linear
      *  in the size of the heap.
      *  Throw a HeapException if the heap is empty. */
    public E poll() {
        // TODO 3: Do poll and bubbleDown together.
        // Our solution to Heap will be available on the Piazza on the morning of 23 April.
        return null;
    }
    
    /** Change the priority of element e to p.
      *  The expected time is logarithmic and the worst-case time is linear
      *  in the size of the heap.
      *  Throw an IllegalArgumentException if e is not in the heap. */
    public void changePriority(E e, double p) {
        // TODO  4: Do updatePriority.
        // Our solution to Heap will be available on the Piazza on the morning of 23 April.
    }
    
    
    /** Bubble b[k] up in heap to its right place.
      *  Precondition: Priority of every b[i] >= its parent's priority
      *                except perhaps for b[k] */
    private void bubbleUp(int k) {
        // TODO  Do add and bubbleUp together.
    //  Our solution to Heap will be available on the Piazza on the morning of 23 April.
    
    }

/** Bubble b[k] down in heap until it finds the right place.
  *  Precondition: Each b[i]'s priority <= its childrens' priorities 
  *                except perhaps for b[k] */
private void bubbleDown(int k) {
    // TODO 3: Do poll and bubbleDown together.
    
    // Throughout, k, bk, and bkInfo describe element k
    //  Our solution to Heap will be available on the Piazza on the morning of 23 April.
}

/** Return the index of the smaller child of b[n]
  *  If the two children have have the same priority, choose the right one.
  *  Precondition: left child exists: 2n+1 < size of heap */
private int smallerChildOf(int n) {
    //Our solution to Heap will be available on the Piazza on the morning of 23 April.
    return -1;
}

/** An instance contains the priority and index an element of the heap. */
private static class Prindex {
    private int index;  // index of this element in map
    private double priority; // priority of this element
    
    /** Constructor: an instance in b[i] with priority p. */
    private Prindex(int i, double p) {
        index= i;
        priority= p;
    }
    
}
}